from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class NFCorpusQueryInstance:
    id : str
    text : str
    url : str

@dataclass
class NFCorpusDocInstance:
    id : str
    text : str
    title : str
    url : str

@dataclass
class NFCorpusQrelInstance:
    qid : str
    did : str
    score : int

class NFCorpusDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/nfcorpus', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> NFCorpusQueryInstance:
        return NFCorpusQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> NFCorpusDocInstance:
        return NFCorpusDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> NFCorpusQrelInstance:
        return NFCorpusQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'NFCorpusDataLoader()'

    def __repr__(self) -> str:
        return 'NFCorpusDataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class ArguanaQueryInstance:
    id : str
    text : str

@dataclass
class ArguanaDocInstance:
    id : str
    text : str
    title : str

@dataclass
class ArguanaQrelInstance:
    qid : str
    did : str
    score : int

class ArguanaDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/arguana', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> ArguanaQueryInstance:
        return ArguanaQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> ArguanaDocInstance:
        return ArguanaDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> ArguanaQrelInstance:
        return ArguanaQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'ArguanaDataLoader()'

    def __repr__(self) -> str:
        return 'ArguanaDataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class TrecNewsQueryInstance:
    id : str
    text : str

@dataclass
class TrecNewsDocInstance:
    id : str
    text : str
    title : str
    url : str
    author : str
    type : str
    source : str
    published_date : int

@dataclass
class TrecNewsQrelInstance:
    qid : str
    did : str
    score : int

class TrecNewsDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/trec-news', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> TrecNewsQueryInstance:
        return TrecNewsQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> TrecNewsDocInstance:
        return TrecNewsDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> TrecNewsQrelInstance:
        return TrecNewsQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'TrecNewsDataLoader()'

    def __repr__(self) -> str:
        return 'TrecNewsDataLoader()'



"""This module provides two functions, _pprint and pprint, for pretty-printing Python objects. 
The _pprint function takes an object and an optional maximum depth, and returns a list of strings that represent a pretty-printed version of the object. It handles lists, dictionaries, and other types of objects differently, creating a tabular representation for lists and dictionaries.
The pprint function uses _pprint to generate a pretty-printed representation of an object, and then prints this representation. Like _pprint, it takes an object and an optional maximum depth as arguments.
Functions:
    _pprint(obj : any, max_deep : int = -1) -> list:
        Returns a pretty representation of the object.
    pprint(obj : any, max_deep : int = -1) -> None:
        Prints a pretty representation of the object.
"""

from dataclasses import asdict
from dataclasses import is_dataclass
import inspect
import io
import os
import types

def _pprint(obj : any, max_item : int = -1, max_deep : int = -1, max_width : int = -1) -> list:
    """Returns a pretty representation of the object.

    Args:
        obj (any): The object to be printed.
        max_item (int, optional): The maximum number of items to be printed. Defaults to -1 (infinite items).
        max_deep (int, optional): The maximum depth of the object to be printed. Defaults to -1 (infinite depth).
        max_width (int, optional): The maximum width of the object to be printed. Defaults to -1 (terminal width).

    Returns:
        list: A list of strings representing the pretty representation of the object.
    """

    if max_width == -1:
        max_width = os.get_terminal_size().columns

    if max_width < 0:
        return ['']

    if max_deep == 0:
        return ['...']

    if obj is None:
        return ['']

    if is_dataclass(obj):
        obj = asdict(obj)

    if type(obj) == list:
        if len(obj) == 0:
            return ['']

        if max_item >= 0 and len(obj) > max_item:
            obj = obj[:max_item]
            obj.append('...')

        col0_width = len(f'{len(obj)}') + 2
        cells = []
        for item in obj:
            cells.append(_pprint(item, max_item, min(max_deep - 1, -1), max_width - col0_width - 5))
        col1_width = 3
        for cell in cells:
            col1_width = max(col1_width, len(cell[0]) + 2)
        row_heights = []
        for cell in cells:
            row_heights.append(len(cell))
        table_width = col0_width + col1_width + 3
        table_height = sum(row_heights) + len(row_heights) + 1

        result = [[' ' for j in range(table_width)] for i in range(table_height)]

        for i in range(table_width):
            result[0][i] = '-'
        result[0][0] = '+'
        result[0][col0_width + 1] = '+'
        result[0][-1] = '+'

        for i in range(len(cells)):
            base_y = sum(row_heights[:i]) + i + 1
            base_x = col0_width + 3
            for y in range(len(cells[i])):
                for x in range(len(cells[i][y])):
                    result[base_y + y][base_x + x] = cells[i][y][x]
            for y in range(row_heights[i]):
                result[base_y + y][0] = '|'
                result[base_y + y][col0_width + 1] = '|'
                result[base_y + y][-1] = '|'
            for j in range(table_width):
                result[base_y + row_heights[i]][j] = '-'
            result[base_y + row_heights[i]][0] = '+'
            result[base_y + row_heights[i]][col0_width + 1] = '+'
            result[base_y + row_heights[i]][-1] = '+'
            result[base_y][1:col0_width] = f'{i:>{col0_width - 1}}'
        return [''.join(row) for row in result]

    elif type(obj) == dict:
        if len(obj) == 0:
            return ['']

        if max_item >= 0 and len(obj) > max_item:
            obj = dict(list(obj.items())[:max_item])
            obj['...'] = '...'

        cells = []
        col0_width = 3
        for key in obj:
            col0_width = max(col0_width, len(f'{key}') + 2)
        for key in obj:
            cells.append(_pprint(obj[key], max_item, min(max_deep - 1, -1), max_width - col0_width - 5))
        col1_width = 3
        for cell in cells:
            col1_width = max(col1_width, len(cell[0]) + 2)
        row_heights = []
        for cell in cells:
            row_heights.append(len(cell))
        table_width = col0_width + col1_width + 3
        table_height = sum(row_heights) + len(row_heights) + 1

        result = [[' ' for j in range(table_width)] for i in range(table_height)]

        for i in range(table_width):
            result[0][i] = '-'
        result[0][0] = '+'
        result[0][col0_width + 1] = '+'
        result[0][-1] = '+'

        for i in range(len(cells)):
            base_y = sum(row_heights[:i]) + i + 1
            base_x = col0_width + 3
            for y in range(len(cells[i])):
                for x in range(len(cells[i][y])):
                    result[base_y + y][base_x + x] = cells[i][y][x]
            for y in range(row_heights[i]):
                result[base_y + y][0] = '|'
                result[base_y + y][col0_width + 1] = '|'
                result[base_y + y][-1] = '|'
            for j in range(table_width):
                result[base_y + row_heights[i]][j] = '-'
            result[base_y + row_heights[i]][0] = '+'
            result[base_y + row_heights[i]][col0_width + 1] = '+'
            result[base_y + row_heights[i]][-1] = '+'
            result[base_y][1:col0_width] = f'{list(obj.keys())[i]:>{col0_width - 1}}'
        return [''.join(row) for row in result]

    elif type(obj) == set:
        return _pprint(sorted(list(obj)), max_item, max_deep, max_width)

    elif isinstance(obj, types.FunctionType):
        my_type = 'Function'
        func_name = obj.__name__
        signature = inspect.signature(obj)
        params = []
        for param in signature.parameters:
            param_type = signature.parameters[param].annotation.__name__
            param_info = f'{param} : {param_type}'
            if signature.parameters[param].default != inspect.Parameter.empty:
                param_info += f' = {signature.parameters[param].default}'
            params.append(param_info)
        info = {'Type': my_type, 'Function': func_name, 'Params': params}
        return _pprint(info, max_item, max_deep, max_width)

    elif isinstance(obj, io.IOBase):
        my_type = 'File'
        file_name = obj.name
        mode = obj.mode
        info = {'Type': my_type, 'File': file_name, 'Mode': mode}
        return _pprint(info, max_item, max_deep, max_width)        

    elif hasattr(obj, '__dict__'):
        if isinstance(obj, type):
            my_type = 'Class'
            class_name = obj.__name__
            
            method_list = []
            for attr in inspect.getmembers(obj):
                if inspect.isfunction(attr[1]) or inspect.ismethod(attr[1]):
                    try:
                        signature = inspect.signature(attr[1])
                    except ValueError:
                        signature = '()'
                    method_info = f'{attr[0]} {signature}'
                    method_list.append(method_info)

            info = {'Type': my_type, 'Class': class_name, 'Methods': method_list}

        elif isinstance(obj, types.ModuleType):
            my_type = 'Module'
            module_name = obj.__name__

            method_list = []
            for attr in inspect.getmembers(obj):
                if inspect.isfunction(attr[1]) or inspect.ismethod(attr[1]):
                    try:
                        signature = inspect.signature(attr[1])
                    except:
                        signature = '()'
                    method_info = f'{attr[0]} {signature}'
                    method_list.append(method_info)

            info = {'Type': my_type, 'Module': module_name, 'Methods': method_list}

        else:
            my_type = 'Class Instance'
            class_name = obj.__class__.__name__

            method_list = []
            for attr in inspect.getmembers(obj.__class__):
                if inspect.isfunction(attr[1]) or inspect.ismethod(attr[1]):
                    try:
                        signature = inspect.signature(attr[1])
                    except ValueError:
                        signature = '()'
                    method_info = f'{attr[0]} {signature}'
                    method_list.append(method_info)

            attributes = obj.__dict__
            var_list = []
            for var in attributes:
                if not var.startswith('__') and not inspect.ismethod(getattr(obj, var)):
                    var_type = type(attributes[var]).__name__
                    max_attr_width = max_width - 11 - len(attributes)
                    var_list.append(f'{var} : {var_type} = {attributes[var]}'[:max_attr_width*3])

            info = {'Type': my_type, 'Class': class_name, 'Methods': method_list, 'Vars': var_list}

        return _pprint(info, max_item, max_deep, max_width)

    else:
        text = str(obj)
        lines = text.replace('\r\n', '\n').split('\n')
        result = []
        for line in lines:
            height = (len(line) + max_width - 1) // max_width
            result += [line[max_width * i: max_width * (i + 1)] for i in range(height)]
        return result

def pprint(obj : any, max_item: int = -1,  max_deep : int = -1, max_width : int = -1) -> None:
    """Prints a pretty representation of the object.

    Args:
        obj (any): The object to be printed.
        max_item (int, optional): The maximum number of items to be printed. Defaults to -1 (infinite items).
        max_deep (int, optional): The maximum depth of the object to be printed. Defaults to -1 (infinite depth).
        max_width (int, optional): The maximum width of the object to be printed. Defaults to -1 (terminal width).
    """
    table = _pprint(obj, max_item, max_deep, max_width)
    text = '\n'.join(table)
    print(text)



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class MSMARCOQueryInstance:
    id : str
    text : str

@dataclass
class MSMARCODocInstance:
    id : str
    text : str

@dataclass
class MSMARCOQrelInstance:
    qid : str
    did : str
    score : int

class MSMARCODataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/msmarco', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> MSMARCOQueryInstance:
        return MSMARCOQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> MSMARCODocInstance:
        return MSMARCODocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> MSMARCOQrelInstance:
        return MSMARCOQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'MSMARCODataLoader()'

    def __repr__(self) -> str:
        return 'MSMARCODataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class TrecCovidQueryInstance:
    id : str
    text : str
    narrative : str

@dataclass
class TrecCovidDocInstance:
    id : str
    text : str
    title : str
    url : str
    pubmed_id : str

@dataclass
class TrecCovidQrelInstance:
    qid : str
    did : str
    score : int

class TrecCovidDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/trec-covid', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> TrecCovidQueryInstance:
        return TrecCovidQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> TrecCovidDocInstance:
        return TrecCovidDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> TrecCovidQrelInstance:
        return TrecCovidQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'TrecCovidDataLoader()'

    def __repr__(self) -> str:
        return 'TrecCovidDataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class FEVERQueryInstance:
    id : str
    text : str

@dataclass
class FEVERDocInstance:
    id : str
    text : str
    title : str

@dataclass
class FEVERQrelInstance:
    qid : str
    did : str
    score : int

class FEVERDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/fever', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> FEVERQueryInstance:
        return FEVERQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> FEVERDocInstance:
        return FEVERDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> FEVERQrelInstance:
        return FEVERQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'FEVERDataLoader()'

    def __repr__(self) -> str:
        return 'FEVERDataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class FiQAQueryInstance:
    id : str
    text : str

@dataclass
class FiQADocInstance:
    id : str
    text : str

@dataclass
class FiQAQrelInstance:
    qid : str
    did : str
    score : int

class FiQADataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/fiqa', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> FiQAQueryInstance:
        return FiQAQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> FiQADocInstance:
        return FiQADocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> FiQAQrelInstance:
        return FiQAQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'FiQADataLoader()'

    def __repr__(self) -> str:
        return 'FiQADataLoader()'



"""This module contains a class BIBExtractor that is used to extract and parse BIB entries from a .bib file. 
The BIBExtractor class has the following methods:
- __init__(self, path: str): Initializes the BIBExtractor instance and imports entries from the .bib file specified by the path.
- import_from_bib(self, path: str): Imports BIB entries from a .bib file.
- _parse_entry_from_raw(self, raw: str) -> dict[str, str]: Parses a raw entry string into a dictionary.
- _parse_entry_from_raw_line(self, raw: str) -> list[str]: Parses lines from a raw entry string, considering nested braces and quotes.
- _clean_entry(self, entry: dict[str, str]) -> dict[str, str]: Cleans and formats an entry values, including converting LaTeX to Unicode.
The BIBExtractor class also has a class variable 'latex_unicode_mapping' which is a dictionary mapping of LaTeX special characters to their Unicode counterparts.
"""

class BIBExtractor:
    """Class to extract and parse BIB entries from a .bib file.
    """
    # Mapping of LaTeX special characters to their Unicode counterparts
    latex_unicode_mapping = {
        '\"a': 'ä', '\"o': 'ö', '\"u': 'ü', '\"A': 'Ä', '\"O': 'Ö', '\"U': 'Ü',
        '\'a': 'á', '\'e': 'é', '\'i': 'í', '\'o': 'ó', '\'u': 'ú', '\'A': 'Á', '\'E': 'É', '\'I': 'Í', '\'O': 'Ó', '\'U': 'Ú',
        '`a': 'à', '`e': 'è', '`i': 'ì', '`o': 'ò', '`u': 'ù', '`A': 'À', '`E': 'È', '`I': 'Ì', '`O': 'Ò', '`U': 'Ù',
        '^a': 'â', '^e': 'ê', '^i': 'î', '^o': 'ô', '^u': 'û', '^A': 'Â', '^E': 'Ê', '^I': 'Î', '^O': 'Ô', '^U': 'Û',
        '~n': 'ñ', '~N': 'Ñ', 'cC': 'Ç', 'cç': 'ç',
    }

    def __init__(self, path : str):
        """Constructor for BIBExtractor

        Args:
            path (str): Path to the .bib file to extract entries from.
        """
        # Initialize an empty list to store entries
        self.entries = []
        # Import entries from a .bib file specified by the path
        self.import_from_bib(path)

    def import_from_bib(self, path : str):
        """Import BIB entries from a .bib file

        Args:
            path (str): Path to the .bib file to import entries from.
        """
        # Flags for state within the file parsing
        is_inside = False  # Flag to check if currently inside a BIB entry
        current_entry_inside = []  # Buffer to store characters of the current entry
        inside_quote = False  # Flag to check if currently inside quotes
        inside_depth = 0  # Nested depth for braces

        with open(path, 'r', encoding='utf-8') as fp:  # Open the .bib file
            for line in fp:  # Iterate through each line
                line = line.lstrip()  # Strip leading whitespace
                if line.startswith('%') or not line:  # Ignore comment and blank lines
                    continue
                for ch in line:  # Iterate through each character
                    # Toggle the inside_quote flag when encountering a quote not preceded by a backslash
                    if ch == '"' and current_entry_inside[-1] != '\\':
                        inside_quote = not inside_quote
                    # Begin a new entry upon encountering '@' outside quotes
                    if ch == '@' and not inside_quote:
                        is_inside = True
                        continue
                    # Increase depth upon encountering '{' outside quotes
                    if ch == '{' and not inside_quote:
                        inside_depth += 1
                    # Decrease depth upon encountering '}' outside quotes
                    if ch == '}' and not inside_quote:
                        inside_depth -= 1
                        # If depth returns to 0, end of entry is reached
                        if inside_depth == 0:
                            is_inside = False
                            # Process and clean the entry, then add to the list
                            current_entry_raw = ' '.join(''.join(current_entry_inside).split())
                            etype, current_entry_raw = current_entry_raw.split('{', 1)
                            current_entry = self._parse_entry_from_raw(current_entry_raw)
                            current_entry['type'] = etype.strip()
                            current_entry = self._clean_entry(current_entry)
                            self.entries.append(current_entry)
                            current_entry_inside = []  # Reset buffer for next entry
                            continue
                    # If inside an entry, add character to buffer
                    if is_inside:
                        current_entry_inside.append(ch)

    def _parse_entry_from_raw(self, raw : str) -> dict[str, str]:
        """Parse a raw entry string into a dictionary

        Args:
            raw (str): Raw entry string to parse.

        Returns:
            dict[str, str]: Parsed entry as a dictionary.
        """
        lines = self._parse_entry_from_raw_line(raw)
        entry = {'id': lines[0]}  # The first line is the ID
        for line in lines[1:]:  # Process each line after the ID
            if not line:
                continue
            key, value = line.split('=', 1)
            entry[key.strip()] = value.strip()
        return entry

    def _parse_entry_from_raw_line(self, raw : str) -> list[str]:
        """Parse lines from a raw entry string, considering nested braces and quotes

        Args:
            raw (str): Raw entry string to parse.

        Returns:
            list[str]: List of parsed lines.
        """
        lines = []
        inside_quote = False
        pivot = 0
        depth = 0
        for i in range(len(raw)):
            if raw[i] == '"' and (i == 0 or raw[i-1] != '\\'):
                inside_quote = not inside_quote
            if inside_quote:
                continue
            elif raw[i] == '{':
                depth += 1
            elif raw[i] == '}':
                depth -= 1
            elif raw[i] == ',' and depth == 0:
                lines.append(raw[pivot:i].strip())
                pivot = i + 1
        lines.append(raw[pivot:].strip())  # Add the last segment
        return lines

    def _clean_entry(self, entry : dict[str, str]) -> dict[str, str]:
        """Clean and format an entry values, including converting LaTeX to Unicode

        Args:
            entry (dict[str, str]): Entry to clean and format.

        Returns:
            dict[str, str]: Cleaned and formatted entry.
        """
        for key in entry:
            entry[key] = entry[key].strip('"{}')
            for k, v in self.latex_unicode_mapping.items():
                # Replace LaTeX encoded characters with their Unicode counterparts
                entry[key] = entry[key].replace('{\\' + k + '}', v)
                entry[key] = entry[key].replace(k, v)
            if key in ['author', 'editor']:
                # Split authors/editors into a list, removing the surrounding braces
                entry[key] = entry[key][1:-1].split(' and ')
        return entry



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class HotpotQAQueryInstance:
    id : str
    text : str

@dataclass
class HotpotQADocInstance:
    id : str
    text : str
    title : str
    url : str

@dataclass
class HotpotQAQrelInstance:
    qid : str
    did : str
    score : int

class HotpotQADataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/hotpotqa', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> HotpotQAQueryInstance:
        return HotpotQAQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> HotpotQADocInstance:
        return HotpotQADocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> HotpotQAQrelInstance:
        return HotpotQAQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'HotpotQADataLoader()'

    def __repr__(self) -> str:
        return 'HotpotQADataLoader()'



from .arguana_data_loader import ArguanaDataLoader
from .bioasq_data_loader import BioASQDataLoader
from .climate_fever_data_loader import ClimateFEVERDataLoader
from .dbpedia_data_loader import DBPediaDataLoader
from .fever_data_loader import FEVERDataLoader
from .fiqa_data_loader import FiQADataLoader
from .hotpotqa_data_loader import HotpotQADataLoader
from .msmarco_data_loader import MSMARCODataLoader
from .nfcorpus_data_loader import NFCorpusDataLoader
from .nq_data_loader import NQDataLoader
from .quora_data_loader import QuoraDataLoader
from .robust04_data_loader import Robust04DataLoader
from .scidocs_data_loader import SciDocsDataLoader
from .scifact_data_loader import SciFactDataLoader
from .signal1m_data_loader import Signal1MDataLoader
from .touche_data_loader import ToucheDataLoader
from .touche_v2_data_loader import Touchev2DataLoader
from .trec_covid_data_loader import TrecCovidDataLoader
from .trec_news_data_loader import TrecNewsDataLoader
import requests

class DataLoader:
    """DataLoader class to load dataset from the list of available datasets
    """
    def __init__(self):
        pass

    @staticmethod
    def dataset_list() -> list:
        """Fetch the list of available datasets from the server.

        Raises:
            Exception: If failed to fetch the dataset list.

        Returns:
            list: List of available datasets.
        """
        res = requests.get('http://research.hamu.me/dataset/api/get_list/')
        if res.status_code == 200:
            data = res.json()
            dataset_list = data['dataset_list']
            return dataset_list
        else:
            raise Exception('Failed to fetch dataset list')

    @staticmethod
    def load(dataset_name : str, *args, **kwargs) -> ArguanaDataLoader | BioASQDataLoader | ClimateFEVERDataLoader | DBPediaDataLoader | FEVERDataLoader | FiQADataLoader | HotpotQADataLoader | MSMARCODataLoader | NFCorpusDataLoader | NQDataLoader | QuoraDataLoader | Robust04DataLoader | SciDocsDataLoader | SciFactDataLoader | Signal1MDataLoader | ToucheDataLoader | Touchev2DataLoader | TrecCovidDataLoader | TrecNewsDataLoader:
        """Load the dataset with the given name.

        Args:
            dataset_name (str): Name of the dataset to load.

        Raises:
            Exception: If the dataset is not found.

        Returns:
            @@DataLoader: The dataset wrapper object.
        """
        dataset_list = DataLoader.dataset_list()
        wrapper_mapping = { dataset['name']: dataset['wrapper'] for dataset in dataset_list }

        if dataset_name in wrapper_mapping:
            wrapper_class = globals()[wrapper_mapping[dataset_name]]
            return wrapper_class(*args, **kwargs)
        else:
            raise Exception(f'Dataset [{dataset_name}] not found')



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class DBPediaQueryInstance:
    id : str
    text : str

@dataclass
class DBPediaDocInstance:
    id : str
    text : str
    title : str
    url : str

@dataclass
class DBPediaQrelInstance:
    qid : str
    did : str
    score : int

class DBPediaDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/dbpedia', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> DBPediaQueryInstance:
        return DBPediaQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> DBPediaDocInstance:
        return DBPediaDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> DBPediaQrelInstance:
        return DBPediaQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'DBPediaDataLoader()'

    def __repr__(self) -> str:
        return 'DBPediaDataLoader()'



from .data_loader_base import DataLoaderQDRBase
from dataclasses import dataclass
from typing import Iterator

@dataclass
class NQQueryInstance:
    id : str
    text : str

@dataclass
class NQDocInstance:
    id : str
    text : str
    title : str

@dataclass
class NQQrelInstance:
    qid : str
    did : str
    score : int

class NQDataLoader(DataLoaderQDRBase):
    def __init__(self, *args, **kwargs):
        super().__init__('beir/nq', *args, **kwargs)

    def make_query_instance(self, **kwargs) -> NQQueryInstance:
        return NQQueryInstance(**kwargs)

    def make_doc_instance(self, **kwargs) -> NQDocInstance:
        return NQDocInstance(**kwargs)

    def make_qrel_instance(self, **kwargs) -> NQQrelInstance:
        return NQQrelInstance(**kwargs)

    def __str__(self) -> str:
        return 'NQDataLoader()'

    def __repr__(self) -> str:
        return 'NQDataLoader()'
